use cgmath::{Vector3, Zero};
use utilities::{impl_reprc, prelude::*};
use vulkan_rs::prelude::*;

impl_reprc!(
    #[derive(Copy, Debug)]
    pub struct PositionNormal {
        #[assume_reprc]
        pub position: Vector3<f32>,

        #[assume_reprc]
        filler_p: f32,

        #[assume_reprc]
        pub normal: Vector3<f32>,

        #[assume_reprc]
        filler_n: f32,
    }
);

impl PositionNormal {
    pub fn new(position: Vector3<f32>, normal: Vector3<f32>) -> Self {
        Self {
            position: position,
            filler_p: 1.0,
            normal: normal,
            filler_n: 0.0,
        }
    }
}

impl Default for PositionNormal {
    fn default() -> Self {
        PositionNormal {
            position: Vector3::zero(),
            filler_p: 1.0,
            normal: Vector3::zero(),
            filler_n: 0.0,
        }
    }
}

impl VertexInputDescription for PositionNormal {
    fn attributes() -> Vec<VkVertexInputAttributeDescription> {
        vec![
            // position
            VkVertexInputAttributeDescription {
                location: 0,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 0,
            },
            // normal
            VkVertexInputAttributeDescription {
                location: 1,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 16,
            },
        ]
    }
}
