#version 450

#ifndef FALLBACK
    #extension GL_EXT_nonuniform_qualifier : require
#endif // FALLBACK

#ifndef AMBIENT
    #define AMBIENT 0.05
#endif

// input parameter
layout (location = 0) in vec2 in_uv;

// output parameter
layout (location = 0) out vec4 out_frag_color;

layout (set = 0, binding = 0) uniform sampler2D sampler_position;
layout (set = 0, binding = 1) uniform sampler2D sampler_normal;
layout (set = 0, binding = 2) uniform sampler2D sampler_albedo;

layout (set = 0, binding = 3) uniform SceneInfo {
    uint point_light_count;
    uint direction_light_count;
    uint spot_light_count;
    vec3 view_position;
} scene_info;

#ifdef FALLBACK
    #if POINT_LIGHT_COUNT > 0
        layout (set = 1, binding = 0) uniform PointLight {
            vec4 position;
            vec3 color;
            float distance;
            mat4 view;
        } point_light_infos[POINT_LIGHT_COUNT];

        layout (set = 2, binding = 0) uniform sampler2D point_light_shadows[POINT_LIGHT_COUNT];
    #endif // POINT_LIGHT_COUNT > 0

    #if DIRECTIONAL_LIGHT_COUNT > 0
        layout (set = 3, binding = 0) uniform DirectionalLight {
            vec4 direction;
            vec3 color;
            float distance;
            mat4 view;
            vec4 light_pos;
        } directional_light_infos[DIRECTIONAL_LIGHT_COUNT];

        layout (set = 4, binding = 0) uniform sampler2D directional_light_shadows[DIRECTIONAL_LIGHT_COUNT];
    #endif // DIRECTIONAL_LIGHT_COUNT > 0

    #if SPOT_LIGHT_COUNT > 0
        layout (set = 5, binding = 0) uniform SpotLight {
            vec4 position;
            vec3 direction;
            float angle;
            vec3 color;
            float distance;
            mat4 view;
        } spot_light_infos[SPOT_LIGHT_COUNT];

        layout (set = 6, binding = 0) uniform sampler2D spot_light_shadows[SPOT_LIGHT_COUNT];
    #endif // SPOT_LIGHT_COUNT > 0
#else
    layout (set = 1, binding = 0) uniform PointLight {
        vec4 position;
        vec3 color;
        float distance;
        mat4 view;
    } point_light_infos[ ];

    layout (set = 2, binding = 0) uniform sampler2D point_light_shadows[ ];

    layout (set = 3, binding = 0) uniform DirectionalLight {
        vec4 direction;
        vec3 color;
        float distance;
        mat4 view;
    } directional_light_infos[ ];

    layout (set = 4, binding = 0) uniform sampler2D directional_light_shadows[ ];

    layout (set = 5, binding = 0) uniform SpotLight {
        vec4 position;
        vec3 direction;
        float angle;
        vec3 color;
        float distance;
        mat4 view;
    } spot_light_infos[ ];

    layout (set = 6, binding = 0) uniform sampler2D spot_light_shadows[ ];
#endif // FALLBACK

bool texture_proj(vec4 P, sampler2D tex)
{
	vec4 shadow_coord = P / P.w;
	shadow_coord.st = shadow_coord.st * 0.5 + 0.5;

	if (shadow_coord.z > -1.0 && shadow_coord.z < 1.0)
	{
		float dist = texture(tex, shadow_coord.st).r;

		if (shadow_coord.w > 0.0 && dist < shadow_coord.z)
		{
			return false;
		}
	}

	return true;
}

void main() {
    // Get G-Buffer values
	vec4 pos = texture(sampler_position, in_uv);

    vec3 frag_pos = pos.xyz;
	vec3 normal = texture(sampler_normal, in_uv).rgb;
	vec4 albedo = texture(sampler_albedo, in_uv);
    float metallic_factor = pos.w;

    // Ambient part
	vec3 frag_color = albedo.rgb * AMBIENT;

    // point lights
    for (int i = 0; i < scene_info.point_light_count; i++) {

    }

    // directional lights
    #ifdef FALLBACK
        #if DIRECTIONAL_LIGHT_COUNT > 0
            for (int i = 0; i < DIRECTIONAL_LIGHT_COUNT; i++) {
                vec4 shadow_clip = directional_light_infos[i].view * vec4(frag_pos, 1.0);

                bool shadowed = texture_proj(shadow_clip, directional_light_shadows[i]);

                if (shadowed) {
                    frag_color *= 0.5;
                }
            }
        #endif // DIRECTIONAL_LIGHT_COUNT
    #else
        for (int i = 0; i < scene_info.direction_light_count; i++) {
            vec4 shadow_clip = directional_light_infos[nonuniformEXT(i)].view * vec4(frag_pos, 1.0);

            bool shadowed = texture_proj(shadow_clip, directional_light_shadows[nonuniformEXT(i)]);

            if (shadowed) {
                frag_color *= 0.5;
            }
        }
    #endif // FALLBACK

    // spot lights
    for (int i = 0; i < scene_info.spot_light_count; i++) {

    }

    // for (int i = 0; i < push_constants.light_count; i++) {
    //     Light light = ubo.lights[i];

    //     // Vector to light
	// 	vec3 L = light.position.xyz - frag_pos;
	// 	// Distance from light to fragment position
	// 	float dist = length(L);

	// 	// Viewer to fragment
	// 	vec3 V = scene_info.view_position - frag_pos;
	// 	V = normalize(V);

    //     // Light to fragment
    //     L = normalize(L);

    //     // Attenuation
    //     float atten = light.radius / (pow(dist, 2.0) + 1.0);

    //     // Diffuse part
    //     vec3 N = normalize(normal);
    //     float NdotL = max(0.0, dot(N, L));
    //     vec3 diff = light.color * albedo.rgb * NdotL * atten;

    //     // Specular part
    //     // Specular map values are stored in alpha of albedo mrt
    //     vec3 R = reflect(-L, N);
    //     float NdotR = max(0.0, dot(R, V));
    //     vec3 spec = light.color * albedo.a * pow(NdotR, 16.0) * atten;

    //     frag_color += (diff + spec) * metallic_factor;
    // }

    out_frag_color = vec4(frag_color, 1.0);
}